/*
** Termine v1.0.1
**
** A WWW calendar entry tool
**
** Copyright (c) 14.6.94 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.08.07
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version history
**
** Version 1.0.1 - 7.12.94
**	Fixed bug resulting in ERROR_BUILD
**
** Version 1.0 - 14.6.94
**	Initial version
*/

char header[]="termine v1.0.1\n(c) 1994 by Andreas Ley\n";

/* General definitions */

#define	TZ	"MEZ-1MESZ"
#define	MASTER	"webmaster@rz.uni-karlsruhe.de"

/* File locations */

#define	ROOT	"/usr/local/etc/httpd/lib/termine"
#define	ACCESS	".access"
#define	BUILDER	"/usr/local/etc/httpd/bin/termine"
#define	INDEX	"/Termine/"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <regex.h>
#include <time.h>
#include "termine_german.h"

#ifdef NEEDSTRERROR
extern int	sys_nerr;
extern char	*sys_errlist[];
#define	strerror(errno)	((errno)>=sys_nerr?"Undefined error":sys_errlist[errno])
#endif /* NEEDSTRERROR */

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

/* Functions from wildmat.c */

extern int wildmat();

/* Functions from util.c */

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
void unescape_url(char *url);
void plustospace(char *str);

/* Global variables */

char	root[]=ROOT;
char	master[]=MASTER;
char	*weekday[]={"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"};

/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s%s%s</TITLE>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"");
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",master);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
	(void)printf("<H1>%s</H1>\n",subtitle?subtitle:TITLE);
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer()
{
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(ERROR,error);
	show_trailer();
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer();
	exit(0);
}



void crtospace(char *ptr)
{
	do
		if (*ptr=='\n')
			*ptr=' ';
	while (*++ptr);
}



int tagindex(char **htag, int max, char *tagname)
{
	int	cnt;

	for (cnt=0;cnt<max&&strcmp(htag[cnt],tagname);cnt++);
	return(cnt);
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(int argc, char *argv[])
{
	int		c;
	char		buffer[1024],*remote_host,*script_name,*tag,*val,*ptr;
	char		*htag[1024],*hval[1024];
	FILE		*src,*dest;
	int		hasaccess,cl,max,day,month,year,tim;
	time_t		tloc;
	struct tm	*timeptr;

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	if(chdir(root))
		show_fatal(ERROR_DIRECTORY);
	if ((src=fopen(ACCESS,"r"))) {
		remote_host=getenv("REMOTE_HOST");
		hasaccess=FALSE;
		while(!hasaccess&&fgets(buffer,sizeof(buffer),src)) {
			ptr=strchr(buffer,'\0');
			if (ptr[-1]=='\n')
				ptr[-1]='\0';
			if (wildmat(remote_host,buffer))
				hasaccess=TRUE;
		}
		(void)fclose(src);
		if (!hasaccess)
			show_error(NO_ACCESS);
	}
	else
		show_fatal(ERROR_READ);

	(void)putenv("TZ=" TZ);
	(void)time(&tloc);
	timeptr=localtime(&tloc);

	if (!strcmp(getenv("REQUEST_METHOD"),"GET")) {
		show_header(NULL,FORM_DESC);
		(void)printf(FORM,INDEX);
		script_name=getenv("SCRIPT_NAME");
		(void)printf(FORM_BODY,script_name,timeptr->tm_year+1900);
		show_trailer();
	}
	else if (!strcmp(getenv("REQUEST_METHOD"),"POST")) {
		if (strcmp(getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded"))
			show_fatal(ERROR_CONTENT_TYPE);

		/* Parse form */
		cl=atoi(getenv("CONTENT_LENGTH"));
		if (cl==0)
			show_fatal(ERROR_CONTENT_LENGTH);
		max=0;
		while (cl&&(!feof(stdin))) {
			val=fmakeword(stdin,'&',&cl);
			plustospace(val);
			unescape_url(val);
			crtospace(val);
			tag=makeword(val,'=');
			htag[max]=strdup(tag);
			hval[max]=strdup(val);
			max++;
		}

		day=atoi(hval[tagindex(htag,max,"day")]);
		month=atoi(hval[tagindex(htag,max,"month")]);
		year=atoi(hval[tagindex(htag,max,"year")]);
		tim=atoi(hval[tagindex(htag,max,"time")]);
		if (day<1||day>31||month<1||month>12||year<94||tim<0||tim>2400)
			show_error(DATE_ILL);
		(void)sprintf(buffer,"%d",day);
		ptr=hval[tagindex(htag,max,"day")];
		while (*ptr=='0')
			ptr++;
		if (strcmp(buffer,ptr))
			show_error(DATE_ILL);
		(void)sprintf(buffer,"%d",month);
		ptr=hval[tagindex(htag,max,"month")];
		while (*ptr=='0')
			ptr++;
		if (strcmp(buffer,ptr))
			show_error(DATE_ILL);
		(void)sprintf(buffer,"%d",year);
		ptr=hval[tagindex(htag,max,"year")];
		while (*ptr=='0')
			ptr++;
		if (strcmp(buffer,ptr))
			show_error(DATE_ILL);
		if (!*hval[tagindex(htag,max,"time")])
			tim=-1;
		else {
			(void)sprintf(buffer,"%d",tim);
			ptr=hval[tagindex(htag,max,"time")];
			while (*ptr=='0')
				ptr++;
			if (strcmp(buffer,ptr))
				show_error(DATE_ILL);
		}
		if (year<100)
			year+=1900;

		if (year<timeptr->tm_year+1900 ||
			(year==timeptr->tm_year+1900 && month<timeptr->tm_mon+1) ||
			(year==timeptr->tm_year+1900 && month==timeptr->tm_mon+1 && day<timeptr->tm_mday))
			show_error(DATE_PAST);

		timeptr->tm_year=year-1900;
		timeptr->tm_mon=month-1;
		timeptr->tm_mday=day;
		tloc=mktime(timeptr);
		timeptr=localtime(&tloc);
		if (year!=timeptr->tm_year+1900 || month!=timeptr->tm_mon+1 || day!=timeptr->tm_mday)
			show_error(DATE_ILL);
			
		if (!*hval[tagindex(htag,max,"title")])
			show_error(NO_TITLE);

		(void)umask(7);
		(void)sprintf(buffer,"%04d%02d%02d",year,month,day);
		if ((dest=fopen(buffer,"a"))) {
			(void)sprintf(buffer,"%04d%04d",tim,tim);
			buffer[2]='\0';
			(void)fprintf(dest,"%s:%s ",tim<0?"??":buffer,tim<0?"??":buffer+6);
			if (*hval[tagindex(htag,max,"url")])
				(void)fprintf(dest,"<EM><A HREF=\"%s\">%s</A></EM>",hval[tagindex(htag,max,"url")],hval[tagindex(htag,max,"title")]);
			else
				(void)fprintf(dest,"<EM>%s</EM>",hval[tagindex(htag,max,"title")]);
			if (*hval[tagindex(htag,max,"desc")])
				(void)fprintf(dest,", %s",hval[tagindex(htag,max,"desc")]);
			(void)fprintf(dest,"<BR>\n");
			(void)fclose(dest);
		}
		else
			show_fatal(ERROR_WRITE);

		if (system(BUILDER))
			show_fatal(ERROR_BUILD);

		/* Drop the user a message */
		show_header(SUCCESS_HEADER,SUCCESS_DESC);
		(void)printf(SUCCESS,INDEX);
		(void)printf("<STRONG>%s</STRONG>, %d.%d.%d<BR>\n",weekday[timeptr->tm_wday],day,month,year);
		(void)printf("%s:%s <EM>%s</EM>",tim<0?"??":buffer,tim<0?"??":buffer+6,hval[tagindex(htag,max,"title")]);
		if (*hval[tagindex(htag,max,"desc")])
			(void)printf(", %s",hval[tagindex(htag,max,"desc")]);
		show_trailer();
	}

	else
		show_error(ERROR_REQUEST_METHOD);
	return(0);
}
